/*global $, window*/
$.fn.addLoader = function () {
    var $loader = $("<img/>").attr("src", "static/img/loading_dots.gif").attr("width", "25px").addClass("loader");
    return this.each(function () {
        if (($(this).children(".loader")).length === 0) {
            $(this).append($loader);
        }
    });
};
$.fn.addDoneIcon = function () {
    var $done = $("<img/>").attr("src", "static/img/check.png").attr("width", "20px").addClass("done");
    return this.each(function () {
        if (($(this).children(".done")).length === 0) {
            $(this).append($done);
        }
    });
};

$(function () {
    "use strict";
    var done = false,
        prev_status = "",
        callbacks = {},
        wait_for_status = function(status, callback) {
            if (!(status in callbacks)) {
                callbacks[status] = [];
            }
            callbacks[status].push(callback);
        },
        check_status = function () {
            $.ajax({
                url: "status",
                cache: false,
                dataType: "json",
                method: "GET",
                success: function (data, status, jqXHR) {
                    var callback;
                    // This is a terribly ugly hack to get around the global status in startapp
                    if (data.status === "downloaded" &&
                            !(data.manifest_state === "ok" || data.manifest_state === "missing")) {

                        data.status = 'install_failed';
                    }
                    if (data.status !== prev_status) {
                        var $current_list = $("#actions-" + data.status).addClass("current"),
                            current_list_id = $current_list.attr("listid");
                        if ($current_list.hasClass("error")) {
                            $(".actionList").hide();
                        } else {
                            $(".actionList.error").hide();
                            $(".actionList").each(function () {
                                if ($(this).attr("listid") < current_list_id) {
                                    $(this).removeClass("current");
                                    if ($(this).hasClass("showIfPassed")) {
                                        $(this).slideDown(100);
                                    } else {
                                        $(this).hide();
                                    }
                                    if ($(this).hasClass("hasLoader")) {
                                        $(this).find(".loader").remove();
                                        $(this).children(".working").hide();
                                        $(this).children(".done").show();
                                    }
                                }
                            });
                        }
                        $current_list.slideDown(100);
                        if (data.status === "running") {
                            done = true;
                            setTimeout(function () {
                                var host = window.location.host,
                                    url = "http://" + host + "/stereo";
                                window.open(url);
                                $current_list.find(".loader").remove();
                                $current_list.find("#popup-blocked-href").attr("href", url);
                                $current_list.find(".delay").slideDown();
                            }, 500);
                        }
                        if (data.status === "install_failed") {
                            var $buttons = $current_list.find('button'),
                                $tip = $current_list.find('.tip span'),
                                $resolution = $current_list.find('.resolution span');

                            $buttons.hide();
                            switch(data.manifest_state) {
                                case "invalid_startapp":
                                    $tip.text("Your ACAP version is not supported by the downloaded package.");
                                    $resolution.html("Please install the <b>latest version of ACAP to continue</b>!");
                                    break;
                                case "invalid_firmware":
                                    $tip.html("The camera firmware version is not supported by the downloaded package.");
                                    $resolution.html("Please upgrade to the <b>latest firmware to continue</b>!");
                                    break;
                                case "invalid_startapp_and_firmware":
                                    $tip.html("The downloaded package does not support your ACAP version and camera firmware version.");
                                    $resolution.html("Please upgrade to the <b>latest camera firmware version</b>, \
                                        and install the <b>latest version of the ACAP to continue</b>!");
                                    break;
                                default:
                                case "ok":
                                    $tip.text("There is a possibility the download was corrupt, so you can try re-downloading as well.");
                                    $buttons.show();
                            }
                        }
                        $current_list.children(".working").each(function () {
                            $(this).addLoader();
                        });
                        if(data.status in callbacks) {
                            for (var index in callbacks[data.status]) {
                                callback = callbacks[data.status].pop(index);
                                callback();
                            }
                        }
                        prev_status = data.status;
                    }
                    if (!done) {
                        setTimeout(function () {
                            check_status();
                        }, 4000);
                    }
                },
                error: function(jqXHR, textStatus, error) {
                    $(".actionList").hide();
                    $("#actions-error").show();
                }
            });
        };
    check_status();
    /*
     * Click behaviours
     */
    $("#actions").on("click", "#action-download", function (event) {
        var proxy_data = $("#download-form").serialize();
        event.preventDefault();
        if ($(".current").attr("id") === "actions-not_present") {
            proxy_data = "";
        } else {
            proxy_data = $("#download-form").serialize();
        }
        var $this = $(this);
        $.ajax({
            url: "cmd/download",
            type: "POST",
            data: proxy_data,
            success: function (data, status, jqXHR) {
                data = JSON.parse(data);
                if (data.status === "Ok") {
                    $("#actions-not_present").add("#actions-download_failed").removeClass("current").hide();
                    $("#actions-downloading").addClass("current").show()
                        .find(".working").addLoader().show()
                        .find(".done").hide();
                    prev_status = "";
                    wait_for_status("downloading", function() {
                        wait_for_status("downloaded", function () {
                            $("#actions-downloaded").removeClass("current").hide();
                            $("#actions-installing").addClass("current").show()
                                .find(".working").addLoader().show()
                                .find(".done").hide();
                            prev_status = "";
                            $.ajax({
                                url: "cmd/install",
                                type: "POST",
                                success: function (data, status, jqXHR) {
                                    console.log("Install command sent successfully");
                                },
                                error: function(jqXHR, textStatus, error) {
                                    console.log(error);
                                }
                            });
                        });
                    });
                }
            },
            error: function(jqXHR, textStatus, error) {
                console.log(error);
            }
        });
    });

    $("#actions").on("click", "#action-install", function () {
        var $this = $(this);
        $this.parent().addLoader();
        $this.prop("disabled", true);
        $.ajax({
            url: "cmd/install",
            type: "POST",
            success: function (data, status, jqXHR) {
                wait_for_status("installing", function() {
                    $this.prop("disabled", false);
                    $this.parent().find(".loader").remove();
                });
            },
            error: function(jqXHR, textStatus, error) {
                $this.parent().find(".loader").remove();
                $this.prop("disabled", false);
                console.log(error);
            }
        });
    });

    $("#actions").on("click", "#action-uninstall", function(){
        if(confirm("Are you sure you want to completely remove the application?")){
            $("#overlay").show();
            $.ajax({
                url: "/local/startapp/cmd/purge",
                type: "POST",
                cache: false,
                data: "confirm=yes",
                success: function(){
                    $.when(waitForUninstall()).done(function(){
                        alert("Successfully uninstalled the application.")
                        location.reload();
                    })
                },
                error: function(){
                    alert("Connection error, please try again.");
                    $("#overlay").hide();
                }
            });
        }
    });

    function waitForUninstall(){
        var deferred = $.Deferred();
        var waiting;
        setTimeout(function getStatus(){
            $.ajax({
                url:"/local/startapp/status",
                type: "GET",
                cache: false,
                dataType: "json",
                success: function(data){
                    if(data['status'] === "not_present"){
                        deferred.resolve();
                    }
                    else{
                        waiting = setTimeout(getStatus(),2000);
                    }
                },
                error: function(){
                    waiting = setTimeout(getStatus(),2000);
                }
            });
        },2000);
        /*
        * Wait for 1 minute. If no change in application status is made we abort
        * the wait and refresh the page.
        */
        setTimeout(function(){
            clearTimeout(waiting);
            deferred.reject();
            alert("Uninstall failed, reloading the page.");
            location.reload();
        },60000);
        return deferred;
    }
});
